#!/usr/bin/perl
#	
#	et2html - script to convert my etext format to html and build a TOC
#	version 2.5 of August 1999. Maintainer: Serge Winitzki.
#	http://www.geocities.com/CapeCanaveral/Lab/5735/1/
#
#	Usage: as filter (stdin->stdout) with options, e.g.
#		perl et2html.pl -nokoi -maketoc < file.txt > file.html
#	If using the "-outputfile=" option, do not redirect STDOUT, i.e.
#		perl et2html.pl -nokoi -maketoc -outputfile=file.html < file.txt
#
#	Accepting options:
#
#	-usedt	use <dt>'s for unindented lines (default not to use)
#	-usetable	use <table> tags around individual pieces
#	-noapostrophe	for english text (do not mangle apostrophes)
#	-nokoi	do not include the KOI header (default to include)
#	-usebrace	convert text inside braces {} to boldface (default not)
#	-nobracket	do not convert text inside brackets [] to italics (default to convert)
#	-globalfootnote	number footnotes globally (default local)
#	-maketoc	generate TOC in the file TOC.html (default not to generate)
#	-noquotation	do not include first line quotations in TOC (default to include)
#	-usetarget	use <a target=...> tags in TOC (default not to use)
#	-usetarget=targetname	specify a certain target window name
#	(or else, output file name will be used if available, or else "1")
#	-outputfile=filename	specify the output file name (instead of
#		writing to STDOUT).
#	-tocbase=filename	Use together with -maketoc and -outputfile
#		options to override reference a certain file in the TOC.
#	-baretoc	do not add HTML headers to TOC file (default to add)
#	-title="title"	set title of resulting HTML file and TOC
#	-justify	make paragraphs HTML-justified (default not justified)
#	-range xx	select a range of pieces for output (default all pieces)
#		expects arguments describing the range(s). Outputs headers only when no ranges given.
#		e.g. "-range 5 - 7, 2, 11 - " means 2, 5, 6, 7 and then 11 to the end. Commas are optional; the order of numbers is irrelevant;
#		"--" can be used instead of "-". Behavior with negative numbers is undefined.
#
######################################################################
#
#	To do:
#
#		need to make output more rule based to make backends easier?
#
#		Revision history:
#
#	changes 2.4->2.5:
#		added -usebrace, -tocbase, and -nobracket options
#		allow inline graphics (must be <img ...=...> in one line)
#		changed look of footnote references (no asterisks)
#		convert quintuple TAB separation to an H2 header
#		make the main title an H1 header
#
#	changes 2.3->2.4
#		added -usetable option and polished the formatting
#		more sophisticated alignment of double indented lines (see format_info.txt)
#
#	changes 2.2->2.3
#		TOC.html now output without koi->win conversion (i.e. in KOI)
#		added -baretoc and -title options
#		header/footer comments and <hr>'s
#		bug fixes
#
#	changes 2.1->2.2
#		added option '-outputfile'
#		fixed 'range' option
#		bug fixes
#
#	changes 2.0->2.1
#		select a range of pieces for output
#		changed default from generating a TOC to not generating
#		changed name of TOC file to TOC.html from toc_win.html
#
#	changes 1.1->2.0:
#		recognize *\(\d+\) as footnote and make it a hyperlink
#		added command-line options (lots of them)
#
#	changes 1.0->1.1:
#		everything is now expected in KOI-8
#		but TOC is still in Win encoding (for sorting purposes)!
#
#		the TOC can be sorted by e.g.
#		sort -t'>' +2.0 -bdf < toc_win.html | perl win2koi > sorted.html
#		(but don't forget possible ' " ... &lt; etc. in titles)
######################################################################

#parse options: crude
$options="@ARGV";

#the default is always the second clause
$want_apostrophe= ($options =~ /-noapostroph/) ? "no" : "yes";
$want_koi= ($options =~ /-nokoi/) ? "no" : "yes";
$want_dt= ($options =~ /-usedt/) ? "yes" : "no";
$want_global_footnotes= ($options =~ /-globalfootnote/) ? "yes" : "no";
$want_quotations= ($options =~ /-noquotation/) ? "no" : "yes";
$want_target= ($options =~ /-usetarget/) ? "yes" : "no";
$target_name= ($options =~ /-usetarget=([^ ]+)/) ? "$1" : "1";	#default target name is "1"
$want_justify= ($options =~ /-justify/) ? "yes" : "no";
$want_toc= ($options =~ /-maketoc/) ? "yes" : "no";
$want_range= ($options =~ /-range([\-0-9, ]*[0-9, ])-?[^\-0-9, ]/
			or $options =~ /-range([\-0-9, ]*)$/) ? "yes" : "no";
$range=$1 if ($want_range eq "yes");

$want_baretoc= ($options =~ /-baretoc/) ? "yes" : "no";
$want_title= ($options =~ /-title="(.*)"/) ? "yes" : "no";
$big_title= $1;

$want_tables= ($options =~ /-usetable/) ? "yes" : "no";
$want_braces= ($options =~ /-usebrace/) ? "yes" : "no";
$want_brackets= ($options =~ /-nobracket/) ? "no" : "yes";

$outfilename= ($options =~ /-tocbase=([^ ]+)/) ? "$1" : "";

#output file options
$want_outfile= ($options =~ /-outputfile=([^ ]+)/) ? "yes" : "no";
if ($want_outfile eq "yes") {
	$outfilename=$1	#this will go into TOC
		unless ($outfilename ne "");	# unless want to override the name in TOC
	open(OUTFH, ">$outfilename");
	$OUTFILE = \*OUTFH;
} else {	#by default output to STDOUT
	$outfilename="THAT_FILE.HTML";	#this will go into TOC by default
	$OUTFILE = \*STDOUT;
}
# change targetname to output file name unless specified explicitly
$target_name= ($want_target eq "yes" and $want_outfile eq "yes" and $target_name eq "1") ? $outfilename : $target_name;

# Format of numbers in references: need a fixed number of digits
$format="%03d";

#toc file name: TOC.html in current directory
$toc_file="TOC.html"; $toc_file_k="toc_temp.html";
$hr="<p><hr><p>";
$piece_flag="<!-- piece -->";
$toc_list_type="ul";
$indent_list_type="ul";
$apostrophe= ($want_apostrophe eq "yes") ? "<sup><i>'</i></sup>" : "'";
$paragraph= ($want_dt eq "yes") ? "<dt>" : "";

$separation="<hr noshade width=5%>";

$table_start= ($want_tables eq "yes") ? "<table cellspacing=0 cellpadding=0 noborder><tr><td>" : ""; 
$table_end= ($want_tables eq "yes") ? "</table>" : "";

$start_of_center="<center>";
$end_of_center="</center>";
$start_title="<h3>";
$end_title="</h3>";
$start_title_center="</center><h3>";
$end_title_center="</h3><center>";
$start_supertitle="<h2>";
$end_supertitle="</h2>";
$start_supertitle_center="</center><h2>";
$end_supertitle_center="</h2><center>";
$start_date="<h4 align=right><em>";
$end_date="</em></h4>";
$start_date_center="<h4 align=right><em>";
$end_date_center="</em></h4>";

$link_tag= ($want_target eq "yes") ? "<a target=$target_name href=" : "<a href=";
$comment_head="h5";
$comment_hr="<hr noshade width=30%>";
$indented= ($want_justify eq "yes") ? "<br>" . "&nbsp;" x 6 . " " : "<dd> ";
$more_indented="&nbsp;" x 3;
$parskip=($want_justify eq "yes") ? "<p align=justify>" : "<p>";
#
#format description as regex:
#
#
#	^___(.*)$	beginning of new piece, indexing information
#
$regex_start='^___(.*)$';
#
#
#	^\t\t\t\t([^\t].*)$	date or author information
#
$regex_date='^\t\t\t\t([^\t].*)$';
#
#
#	^\t\t\t([^\t].*)$	title
#
$regex_title='^\t\t\t([^\t].*)$';
$regex_supertitle='^\t\t\t\t\t([^\t].*)$';
#
#
#	^\s*$	empty line - new paragraph; note that
# an untabbed line after a tabbed line does not constitute a new untabbed paragraph -
# it is taken to be the continuation of the previous line,
# as it happens most often in prose. But in poetry, every new line is a new line!
#
$regex_par='^\s*$';
#
#
#	^\t\t([^\t].*)$		double indented line of text
#
$regex_indind='^\t\t([^\t ].*)$';
$regex_indind_space='^\t\t ([^\t ].*)$';
$regex_indind_2space='^\t\t  ([^\t ].*)$';
$regex_indind_3space='^\t\t   ([^\t ].*)$';
#
#
#	^\t([^\t].*)$		single indented line of text
#
$regex_ind='^\t([^\t ].*)$';
$regex_ind_space='^\t ([^\t ].*)$';
#
#
#	^([^\t].*)$		unindented line of text
#
$regex_unind='^([^\t].*)$';
#
#
#
#	^\*\t([^\t].*)$		author's remark or editor's comment
#
$regex_comment='^\*\t([^\t].*)$';
#
#
#	\[(.*)\]		italics in text
#
#$regex_it='\[(.*)\]';	#don't really need it, doing italics by hand
#

#line types:
#		0 - unindented,
#		1 - single indented,
#		2 - double indented,
#		3 - triple indented (title information)
#		4 - quadruple indented (date information)
#		5 - start of piece
#		6 - empty line (new paragraph)
#		7 - unknown (does not match)
#		8 - comment/remark
#		9 - quintuple indented (supertitle)

#$state=0;	#current state of the machine
$current=0;	#current number of the piece

open(TOCFILE, ">$toc_file_k") if ($want_toc eq "yes");

$koi_header= ($want_koi eq "no") ? "" : "<META  HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=KOI8-R\">\n";
$header_start="<head>";
$header_end="</head>\n";
$head_title= ($want_title eq "yes") ? "<title>$big_title</title>\n" : "<title>Text collection</title>";
$text_title= ($want_title eq "yes") ? "<h1 align=center myid=title>$big_title</h1>\n" : "";

$common_header= "$header_start$koi_header$head_title$header_end$text_title$hr<!-- header -->";

print $OUTFILE "$common_header$table_start<$indent_list_type>\n";
print TOCFILE "$common_header<$toc_list_type>\n" if ($want_toc eq "yes" and $want_baretoc eq "no");

$got_title=0;
$center_this=0;

#now the range stuff
if ($want_range eq "yes") {
	my ($next, $state, $current_range)= ("", 1, 0);	#$state=1 if waiting for the first number in range, and 2 if waiting for the second

	foreach $next (split(/[ \t]+/, "$range")) {
		$next =~ s/,//g;	#commas are just ignored
		next if ($next eq "");
		if ($next eq "-" or $next eq "--") {
			$state= 2;
		} else {
			if ($state == 1) {
				$current_range= $next;
			} else {
				$state= 1;
			}
			$range{$current_range}= $next;
		}
	}

	$range{$current_range}= "-1" if ($state == 2);	#i.e. we were left with a hanging "to" mark. -1 is a special value.
}

#main loop on stdin
while(<STDIN>) {
	if ($_ =~ /$regex_start/) {	#need to check for beginning of new segment right away
		++$current;
	}
	next unless ($want_range eq "no" or &in_range());


	chop;
#we'll determine the line type by matching
	$line_type=7;	#unknown until we find it out
	$text="";	#temporary

#transform < and > first
	$_ =~ s/\</\&lt;/g;
	$_ =~ s/\>/\&gt;/g;

#now find if we have mangled any included graphics

	$_ =~ s/\&lt;(img.+=.+)\&gt;/<$1>/gi;

#transform apostrophes: don't transform them in English text!
	$_ =~ s/([^-0-9A-z \.,\":;!?)(])\'/$1$apostrophe/g;
	$_ =~ s/\'([^-0-9A-z \.,\":;!?)(])/$apostrophe$1/g;

#transform italics now - not using regex_it
	if ($want_brackets eq "yes") {
		$_ =~ s/\[/<i>/g;
		$_ =~ s/\]/<\/i>/g;
	}

#transform braces
	if ($want_braces eq "yes") {
		$_ =~ s/\{/<b>/g;
		$_ =~ s/\}/<\/b>/g;
	}

#transform footnotes. We do not allow footnotes in the header line.
	&decide_footnote();
	$_ =~ s|\*\((\d+)\)|<sup><a href=\#$footnote$1><b>$1</b></a></sup>|g;

#have to check the starting point first, since it's unindented text	
	if ($_ =~ /$regex_start/) {
		$text="$1";
		$line_type=5;
		$got_title=0;
		if ($center_this) {
			print $OUTFILE "$end_of_center\n";
		}
		print $OUTFILE "</$indent_list_type>";
		if ($had_fn==0) {	# If already had some footnotes, the closing table tag has already been printed.
			print $OUTFILE "$table_end";
		}
		printf $OUTFILE "$hr$piece_flag<a name=$format></a>$table_start<$indent_list_type>\n", $current;
		$center_this=0;
		$had_fn=0;
		if ($text =~ /center/) {	# the text after ___ contains the word 'center'
			$center_this=1;
			print $OUTFILE "$start_of_center\n";
		}
		printf TOCFILE "\n<li>${link_tag}$outfilename#$format>", $current if ($want_toc eq "yes");
	}
	elsif ($_ =~ /$regex_comment/ ) {
		$text="$1";
		$line_type=8;
		$fn_ref="";
		if ($text =~ /^(\d+)\. (.*)$/) #try to see if it is a numbered footnote
		{
			&decide_footnote();
			$fn_ref="<a name=$footnote$1></a> <sup>$1</sup> ";	# Footnote reference.
			$text="$2";
		}
		++$had_fn;
		if ($had_fn==1) {
			print $OUTFILE "$table_end\n$comment_hr\n";
		}
		print $OUTFILE "<$comment_head>$fn_ref$text</$comment_head>\n";
	}
	elsif ($_ =~ /$regex_unind/ ) {
		$text="$1";
		$line_type=0;
		print $OUTFILE "$paragraph$text\n";	#was <dt>$text here
	}
	elsif ($_ =~ /$regex_ind/) {
		$text="$1";
		$line_type=1;
		print $OUTFILE "$indented$text\n";
	}
	elsif ($_ =~ /$regex_ind_space/) {
		$text="$1";
		$line_type=1;
		print $OUTFILE "$indented$more_indented$text\n";
	}
	elsif ($_ =~ /^\t\t---$/) {	# Separation. Check before checking general double indents.
		$text="$1";
		$line_type=2;
		print $OUTFILE "$separation\n";
	}
	elsif ($_ =~ /^\t\t([^\t ].*)$/) {	# Double indented line.
		$text="$1";
		$line_type=2;
		print $OUTFILE "<dl><dl><dd>$text</dl></dl>\n";
	}
	elsif ($_ =~ /$regex_indind_space/) {
		$text="$1";
		$line_type=2;
		print $OUTFILE "<dl><dl><dd>$text</dl></dl>\n";
	}
	elsif ($_ =~ /$regex_indind_2space/) {
		$text="$1";
		$line_type=2;
		print $OUTFILE "<dl><dl><dd>$more_indented$text</dl></dl>\n";
	}
	elsif ($_ =~ /$regex_indind_3space/) {
		$text="$1";
		$line_type=2;
		print $OUTFILE "<div align=right><i>$text</i></div>\n";
	}
	elsif ($_ =~ /$regex_title/) {
		$text="$1";
		$line_type=3;
		print $OUTFILE (($center_this) ? "$start_title_center$text$end_title_center\n" : "$start_title$text$end_title\n");
		print TOCFILE " $text</a>" if ($want_toc eq "yes");	#note that we insert an extra space here, for sorting purposes
	}
	elsif ($_ =~ /$regex_supertitle/) {
		$text="$1";
		$line_type=9;
		print $OUTFILE (($center_this) ? "$start_supertitle_center$text$end_supertitle_center\n" : "$start_supertitle$text$end_supertitle\n");
		print TOCFILE " $text</a>" if ($want_toc eq "yes");	#note that we insert an extra space here, for sorting purposes
	}
	elsif ($_ =~ /$regex_date/) {
		$text="$1";
		$line_type=4;
		#remove extra parentheses around the date that might be present
		if ($text =~ /^\((.*)\)$/) { $text="$1"; }
		print $OUTFILE (($center_this) ? "$start_date_center$text$end_date_center\n" : "$start_date$text$end_date\n");
		print TOCFILE " ($text)" if ($want_toc eq "yes");
	}
	elsif ($_ =~ /$regex_par/) {
		$line_type=6;
		print $OUTFILE "$parskip\n";
	}
	

#then, decide what to do based on that and our current state
#	if ($state==0) {	#don't have a beginning yet
#		if ($line_type==5) {
#			$state=1;
#		}	#ignore all other line titles
#		else {
			
#		}

		if ($got_title eq 0 and $want_toc eq "yes") {
			if ($line_type eq 1) {
				#need to print some title now
				#we shall put the quotation here and add the parens around it later
				if ($text =~ /^\"*(.*)\"*$/) { $text="$1"; }	#get rid of extra quotes
				if ($text =~ /(.*)[\.,\:\;\- ]+$/) { $text="$1"; }	#get rid of extra punctuation at the end
				if ($text =~ /^[\-\. ]+(.*)$/ ) { $text="$1"; } 	#extra stuff at the beginning
				print TOCFILE "\"$text...\"";	# if ($want_quotations eq "yes");
				#we always print the quotation for now, get rid of it later
				print TOCFILE "</a>";
				$got_title=1;
			} elsif ($line_type eq 0) {
				print TOCFILE "\"$text ";	# if ($want_quotations eq "yes");
			}
		}
#	} elsif ($state==1) {	#in text, but no title yet; expecting title or a text line
#		if ($line_type==5) {	#text ended, wrap up the work
#			$state=1;
#		}
#	}
}	#end of the while loop

print $OUTFILE "<!-- footer --></$toc_list_type>$table_end$hr\n";
close ($OUTFILE);

if ($want_toc eq "yes") {
	close (TOCFILE);

#prepare a nicer toc
	open (TOCFILEK, "$toc_file_k");
	open (TOCFILE, ">$toc_file");
	while(<TOCFILEK>) {
		#add parens around the quotation and generally clean it up
		if ($want_quotations eq "yes") {
			s|</a>(.*)</a>| ($1)</a>|;
			# Clean up stuff around ...
			s/[-,.]+\.\.\./.../;
			s/ +\.\.\./.../;
		} else {	#remove quotations
			s|</a>(.*)</a>|</a>|g;
		}
		print TOCFILE;
	}

	close (TOCFILEK);
	print TOCFILE "\n<!-- footer --></$toc_list_type>$hr\n" if ($want_baretoc eq "no");
	close (TOCFILE);
	unlink "$toc_file_k";	#delete the temporary toc file
}

sub decide_footnote {	# Must set global variable $footnote to the footnote prefix.
	$footnote= ($want_global_footnotes eq "no") ? "fn_${current}_" : "#fn_";
}

sub in_range {	#check that $current is in %range
	my ($lower, $in)= (0, 0);
	
	foreach $lower (keys %range) {
		$in= 1 if ($current >= $lower and ($range{$lower} == "-1" or $current <= $range{$lower}));
	}
	$in;
}
