#!/usr/bin/perl
#
#	script to select certain pieces from an et-file version 1.1
#	Usage: et_select -verbose -range <number> ... - <number> ...
#
#	expect arguments describing the range(s).
#	e.g. "- 1 5 -- 8, 3, 11 -" means "0, 1, 5, 6, 7, 8, 3" and then 11 to the end.
#	Commas are optional. Order of numbers determines the output.
#	Version history:
#	1.0->1.1:
#		options parsing, -verbose, -range
#	0.1->1.0:
#		support arbitrary order of ranges, reading file into memory
#		printing numbers of unused pieces
######################################################################

use strict;
my ($regex_start, $next, @order, $state, $current_range, $current_count, @pieces, @unused, $total_printed, $total_used, $want_verbose, $numbers);
$regex_start='^___(.*)$';

# Read the entire file and parse it into individual pieces.
$current_count= 0;

while(<STDIN>) {
	if (/$regex_start/) {
		++$current_count;
	}
	$pieces[$current_count] .= $_;
	$unused[$current_count] = 1;
}

# Parse the command line and create the array of indices in @order.

$want_verbose= ("@ARGV" =~ /-verbose/) ? 1 : 0;
$numbers= ("@ARGV" =~ /-range([\-0-9, ]*[0-9, ])-?[^\-0-9, ]/
			or "@ARGV" =~ /-range([\-0-9, ]*)$/) ? $1 : "";

$state=1;	#$state=1 if waiting for the first number in range and 2 if waiting for the second number
$current_range=0;
foreach $next (split(" ", $numbers)) {
	$next =~ s/,//g;
	next if ($next eq "");
	if ($next eq "-" or $next eq "--") {	#got range delimiter, first index of the range is $current_range
		$state= 2;
		push(@order, 0) if ($#order == -1);
	} else {	#got a number
		if ($state == 1) {	#we weren't in range, got number $next
			$current_range= $next;
			push(@order, $next);
		} else {	#we were in range. Now our range is $current_range until $next
			$state= 1;
			push(@order, $current_range+1 .. $next);
		}
	}
}

push(@order, $current_range+1 .. $#pieces) if ($state == 2);	#i.e. we were left with a hanging range mark

$total_printed= 0;
foreach $next (@order) {
	print $pieces[$next];
	$unused[$next]= 0;
	++$total_printed;
}

if ($want_verbose) {

	$total_used= 0;
	foreach $next (@unused) {
		++$total_used if ($next == 0);
	}

	printf STDERR "et_select.pl: out of %d pieces, %d used, total %d printed.\n", $#pieces+1, $total_used, $total_printed;

	if ($total_used<$#pieces+1) {
		print STDERR "Numbers for unused piece(s):\n";
		$next=0;
		while ($next <= $#unused) {
			print STDERR "$next " if ($unused[$next]);
			++$next;
		}
		print STDERR "\n";
	}
}
