#!/usr/bin/perl
#
# Read STDIN and munge into STDOUT according to ranges
# We use this once on the main HTML text and then on the TOC files.
# Some contiguous ranges of pieces should be kept in one file, hence the -keep option.
# Default mode: TOC munging, nothing kept.
#
# Options:
#
#	-base=<name>	use <name>###.html for file names (mandatory)
#	-text	interpret STDIN as main text (default: interpret as TOC)
#	-keep [<number> - <number>] ... 	specify contiguous ranges
#	-indent	for TOC mode, indent pieces that are inside files
#
######################################################################
# We are going to put pieces in files with the same numbers (so the files would not be contiguously numbered but who cares).
# For example: -base=aa -keep= 2 - 
#
######################################################################

use strict;

my ($base_name, $text_or_toc, $want_keeps, $want_indent, %range, $footer, $format, $indent, $dedent);
my ($header, $current, $have_header) = ("", 0, 0);

# Footer to be added to individual pieces. Make sure to close the table!
$footer="</ul></table><p><hr>\n";

# Format of numbers in references: need a fixed number of digits. It is important that this be the same as the $format in et2html.pl!
$format="%03d";

# Indentation of TOC items.
$indent="<ul>\n";
$dedent="</ul>\n";

# Read command line and parse kept ranges into a hash.

$base_name= ("@ARGV" =~ /-base=([^ ]+)/) ? $1 : "";
if ($base_name eq "") {
	print STDERR "Error: base name not specified (-base=...), cannot continue.\n";
	exit;
}

$want_indent=("@ARGV" =~ /-indent/i) ? 1 : 0;

$text_or_toc= ("@ARGV" =~ /-text/i) ? "text" : "toc";

$want_keeps= ("@ARGV" =~ /-keep([\-0-9, ]*[0-9, ])-?[^\-0-9, ]/
		or "@ARGV" =~ /-keep([\-0-9, ]*)$/) ? 1 : 0;
if ($want_keeps) {	# It is important to parse the ranges array immediately after matching here.
	my ($next, $state, $current_range)= ("", 1, 0); # $state=1 if waiting for the first number in range, and 2 if waiting for the second

	foreach $next (split(/[ \t]+/, $1)) {
		$next =~ s/,//g;        #commas are just ignored
		next if ($next eq "");
		if ($next eq "-" or $next eq "--") {
			$state= 2;
		} else {
			if ($state == 1) {
				$current_range= $next;
			} else {
				$state= 1;
			}
			$range{$current_range}= $next;
		}
	}
	$range{$current_range}= "-1" if ($state == 2);  #i.e. we were left with a hanging "to" mark. -1 is a special value.
}

# Now, the %range hash contains pairs of range delimiters, with "-1" denoting end of file.

# If in TOC mode: Go through input and replace all found references href=...#...> by href=newname newnumber # number.
# If in text mode: make an array of actual text of pieces. Collect headers and footers. Append a header and a footer to each file when printing.

if ($text_or_toc eq "toc") {	# Indent if we aren't already indented and if our current number is not equal to the file number.
	my ($indented)=(0);

	while (<STDIN>) {
		# Replace the title thing with a reference to file #0. This must be the same as in et2html.pl!
		s/<h2 align=center myid=title>(.*)<\/h2>/"<h2 align=center><a href=$base_name" . &get_file_number(0) . ".html>$1<\/a><\/h2>"/e;
		# Replace reference to a piece.
		s/href=.*html#([0-9]+)>/"href=$base_name" . &get_file_number($1) . ".html#$1>"/ie;
		if (&get_file_number($1) != $1 and not $indented) {	# Should indent now.
			print $indent if ($want_indent);
			$indented=1;
		} elsif (&get_file_number($1) == $1 and $indented) {	# Should dedent now.
			print $dedent if ($want_indent);
			$indented=0;
		}
		print $_;
	}
} elsif ($text_or_toc eq "text") {
	while (<STDIN>) {
		if ($have_header) {	# Invariant: current file is open.
			if (/\<\!-- piece --\>/) {	# Start of new piece.
				++$current;
				# Need to open a new file only if the piece is not kept in a range.
				if (&get_file_number($current) == $current) {
					print CURFILE $footer;
					close CURFILE;
					&open_file();
					print CURFILE $header;
				}
			}	# New piece or not, we need to print the current line.
			print CURFILE $_;
		} else {
			if (/\<\!-- header --\>/) {
				$have_header=1;	# Now we start reading the file proper.
				&open_file();
				print CURFILE $header;
			} else {
				$header .= $_;
			}
		}
	}	# End of while loop.
	close CURFILE;
}

# End.

sub get_file_number {  # Check that argument is in %range and return the corrected number
	my ($lower, $index)= (0, @_);
	# To get the effect of merged ranges, we go through the ranges in descending lower boundary order and assign the current index to be the lower boundary if we are in range. The index will be further lowered if the next range overlaps with the current one etc.
	foreach $lower (sort {$b <=> $a} keys %range) {
		$index= $lower if ($index >= $lower and ($range{$lower} == "-1" or $index <= $range{$lower}));
	}
	sprintf($format, $index);
}

sub open_file {	# Compute the current file name and open it as CURFILE.
	open (CURFILE, ">$base_name" . &get_file_number($current) . ".html");
}

