#!/usr/bin/perl
#
# Transform the TOC into a alphabetically sorted, categorized TOC.
# Version 1.0. March 1999. Maintainer: Serge Winitzki.
# Use as filter (STDIN->STDOUT) on TOC.html files from et2html.pl.
# Features / options:
# Generates sorted TOC.
# -nokoi	Disable KOI-8 conversions and sorting rules
# -nospace	Ignore spaces when sorting (not by default)
# -permute	Generate permuted index
# -nbsp	Add nbsp before titles for clarity
# -indexrus or -indexlat	Generate index entries by letter categories
# -vspace	Add extra vertical space between index entries
# Quirks:
# Ignores punctuation when sorting.
# Russian text is expected and output in KOI-8 encoding.
# Ignores case and preserves href targets verbatim. Preserves
# header/footer.
# Case folding: by hand because //i does not work on higher ASCII.
#
# We assume that the file consists of some headers, then some lines
# matching $item_regex, then some footers. Anything that does not match
# will be thrown into footers.
#
######################################################################

$ruswin="";
$ruskoi="";

# Now we assign the category labels. They should pass inside [] in regex. Russian uses KOI-8 encoding!
$rus_cats="1-9 A-Z                   ";
eval('$rus_cats' . " =~ tr/$ruskoi/$ruswin/");
$lat_cats="1-9 A BC D E F G H IJ K L M NO PQ R S T UV W XYZ";

$want_index= ("@ARGV" =~ /-index(rus|lat)/i) ? 1 : 0;

@kind_of_index= ($1 =~ /rus/i) ? eval("qw($rus_cats)") : ($1 =~ /lat/i) ? eval("qw($lat_cats)") : ();

# Whether to expect KOI-8 and sort accordingly. Default use KOI-8.
$want_koi= ("@ARGV" =~ /-nokoi/i) ? 0 : 1;

# Whether sorting should be done without regard to spaces.
$want_nospace= ("@ARGV" =~ /-nospace/i) ? 1 : 0;

# Whether permuting should be done (only if TOC contains quotations)
$want_permuted= ("@ARGV" =~ /-permute/i) ? 1 : 0;

# Whether &nbsp; will be inserted in front of titles
$want_nbsp= ("@ARGV" =~ /-nbsp/i) ? 1 : 0;
$nbsp= ($want_nbsp) ? "&nbsp;" : " ";

$want_vspace= ("@ARGV" =~ /-vspace/i) ? 1 : 0;
$vspace= ($want_vspace) ? "<p>" : "";

$output="";
$headers="";
$footers="";
$item_regex='^<li> *(<a [^>]+>) *(.*)<\/a>(.*)$';

$item_text_regex='^(.*) \(\"(.*)\.\.\.\"\)$';

$li="<li>";
$ul="ul";
$hr="<p><hr><p>";
$have_items=0;
$index=0;
$top_ref="<h5 align=center><a href=#Top>Top</a></h5>";

# Read input and collect all TOC entries into @toc and %toc_stripped.
# We should sort keys %toc_stripped and use the resulting number sequence as order for @toc
while(<STDIN>) {
	eval ("tr/$ruskoi/$ruswin/") if ($want_koi);
	if (/$item_regex/i) {	# Now looking at an item line, just matched.
		$have_items=1;
		$href=$1; $text=$2; $date=$3;
		$text =~ s/^ +//;	# Don't want extra spaces in titles now.
		$have_separate_quotation=0;
		# Add space if title starts with a non-punctuation character.
		$this_nbsp= ($text =~ /^[^-\'\`\"\.]/) ? $nbsp : "";
		if ($text =~ /$item_text_regex/) {
			$title=$1; $quotation=$2;
			$have_separate_quotation=1;
			$_="$li$this_nbsp$href$title</a> (\"$quotation...\")$date";
		} else { # Don't have separate quotation.
			$_= "$li$this_nbsp$href$text</a>$date";	# Don't add newline.
		}
		++$index;
		$toc[$index]=$_;	# Not using $_ any more.
		$_=$text;
		&strip_for_sort();		# Operate on $_.
		$toc_stripped{$_}=$index;
		# If permuted index is desired, we need to add an extra entry.
		if ($want_permuted and $have_separate_quotation and not $title eq $quotation) {	 #Avoid unnecessary permuted entries.
			++$index;
			# An extra cosmetic touch. Never need nbsp here.
			$toc[$index]="$li\"$quotation...\" ($href$title</a>)$date";
			$_="\"$quotation...\" ($title)";
			&strip_for_sort();		# Operate on $_.
			$toc_stripped{$_}=$index;
		}
	} elsif ($have_items) {	# Now looking at footers.
		$footers .= $_;
	} else {	# Now looking at headers.
		$headers .= $_;
	}
}

$output .= $headers;

if ($want_index) {	# Print the index preamble and then categorize the TOC entries.
	$output .= "</$ul>\n<a name=Top></a>\n<h5 align=center>\n";
	foreach $item (@kind_of_index) {
		$newitem=$item;
		$newitem =~ s/([^- ])([^- ])/$1 $2/g;
		$newitem =~ s/([^- ])([^- ])/$1 $2/g;
		$output .= "<a href=#$item>$newitem</a>\n";
	}
	$output .= "</h5>\n$hr\n<$ul>\n";	# End of index and start of entries list.
	
	$cat_index=0;
	&print_cat_name();
	
	foreach $item (sort keys %toc_stripped) {
		# If the item matches the current category, we print it as before. (The current category's label has already been printed.)
		# If it does not match, we go through categories printing their labels until we find the category it matches, and then we print the item as before.
		while (not $item =~ /^[$kind_of_index[$cat_index]]/ and $cat_index < $#kind_of_index) {
			++$cat_index;
			&print_cat_name();
		}
		$output .= $vspace . $toc[$toc_stripped{$item}] . "\n";	# Because now we are using constructed TOC entries.
	}
		# After all items are printed, we print the category labels for categories that are left.
	if ($cat_index < $#kind_of_index) {
		while ($cat_index < $#kind_of_index) {
			++$cat_index;
			&print_cat_name();
		}
	}
	$output .= $footers;

} else {
	foreach $item (sort keys %toc_stripped) {
		$output .= $vspace . $toc[$toc_stripped{$item}] . "\n";
	}

	$output .= $footers;
}

eval('$output' . " =~ tr/$ruswin/$ruskoi/;");
print $output;

# End.

sub print_cat_name {	# Print the category label using current values.
	my ($item, $newitem);
	$item=$kind_of_index[$cat_index];
	$newitem=$item;
	$newitem =~ s/([^- ])([^- ])/$1 $2/g;	# Split range into letters.
	$newitem =~ s/([^- ])([^- ])/$1 $2/g;
	$output .= "\n<a name=$item></a>\n$top_ref<$ul><h4>$newitem</h4></$ul>\n\n";
}

sub strip_for_sort {	# Operates on $_.
	s/^ +//;
	s/^\&lt\; *//;
	s/[\.,:;!\?\'\`\"]//g;
	s/ //g if ($want_nospace);
	if ($want_koi) {
		eval("tr/$ruswin/$ruskoi/");
				tr/ţ// ;
		eval("tr/$ruskoi/$ruswin/");
	}
}
